#!/usr/bin/python3

import json, sys, sqlite3

def main():
	with sqlite3.connect(sys.argv[1]) as db:
		db.text_factory = decode_str
		reports = read_inclusion_reports_map(db)

	for key in sorted(reports.keys()):
		print("=== Thing {} ===".format(key))
		print("node_id = %s , product = %s"%(reports[key]["val"]["address"],reports[key]["val"]["product_hash"]))
        # print(dump_json(reports[key]))
		
def read_inclusion_reports_map(db):
	result = {}

	for thing_id in read_thing_ids(db):
		result[thing_id] = {}

	for thing_id, report in read_inclusion_reports(db):
		result[thing_id] = report

	return result
	
def read_inclusion_reports(db):
	cur = db.cursor()
	try:
		cur.execute("SELECT thingId, inclusionReport FROM Thing ")
		return ((str(prop[0]), load_json(prop[1])) for prop in cur)
	except:
		return tuple()

def read_thing_ids(db):
	cur = db.cursor()
	try:
		cur.execute("SELECT thingId FROM Thing")
		return (str(thing[0]) for thing in cur)
	except:
		return tuple()

def load_json(s):
	try:
		return json.loads(s)
	except:
		return None

def dump_json(obj):
	if obj:
		return json.dumps(obj, indent=2)
	else:
		return ""

def decode_str(x):
	try:
		return str(x, 'utf8')
	except:
		return bytes(x).hex()

if __name__ == "__main__":
	main()