import binascii
import time
from ziptools.zip_client import ZipClient
from fimp.mqtt_transport import MqttTransport
from fimp.message import Message, ValueType

# IMAClient request IMA information from zipgateway , parses and reports it over FIMP

COMMAND_CLASS_NETWORK_MANAGEMENT_INSTALLATION_MAINTENANCE = 0x67
STATISTICS_REPORT = 0x05
STATISTICS_CLEAR  = 0x06
PRIORITY_ROUTE_GET = 0x02
PRIORITY_ROUTE_REPORT = 0x03
# ENUM
ROUTE_CHANGES = 0
TRANSMISSION_COUNT = 1
NEIGHBORS = 2
PACKET_ERROR_COUNT = 3
TRANSMISSION_TIME_SUM = 4
TRANSMISSION_TIME_SUM2 = 5

# ENUM

IMA_NODE_SPEED_96 = 1
IMA_NODE_SPEED_40 = 2
IMA_NODE_SPEED_100 = 3
IMA_NODE_SPEED_200 = 4
IMA_NODE_SPEED_777 = 7
IMA_NODE_REPEATER = 0x80

LINK_SPEED_MAP = {
    IMA_NODE_SPEED_96: 9,
    IMA_NODE_SPEED_40: 40,
    IMA_NODE_SPEED_100: 100,
    IMA_NODE_SPEED_200: 200,
    IMA_NODE_SPEED_777: 0
}

'''

static command_handler_codes_t ima_CommandHandler(zwave_connection_t *c, BYTE* pData, WORD bDatalen) {
  int i;
  BYTE routeType;

  switch(pData[1]) {
    case PRIORITY_ROUTE_SET:
      if(bDatalen >=8) {
      ZW_SetLastWorkingRoute(pData[2],&pData[3]);
      }
      break;
    case PRIORITY_ROUTE_GET:

      pData[1] = PRIORITY_ROUTE_REPORT;
//      pData[2] = pData[2] nodeID;
      routeType = ZW_GetPriorityRoute(pData[2],&pData[4]);
      pData[3] = routeType;
      ZW_SendDataZIP(c,pData,6+3,0);
      break;
    case STATISTICS_GET:
    {
      u8_t *p = pData;
      const struct node_ima* ima = ima_get_entry(pData[2]);

      *p++ = COMMAND_CLASS_NETWORK_MANAGEMENT_INSTALLATION_MAINTENANCE;
      *p++ = STATISTICS_REPORT;
      *p++ = pData[2]; //NodeID
      *p++ = TRANSMISSION_COUNT;
      *p++ = 1;
      *p++ = ima->tc;

      *p++ = PACKET_ERROR_COUNT;
      *p++ = 1;
      *p++ = ima->pec;

      *p++ = ROUTE_CHANGES;
      *p++ = 1;
      *p++ = ima->rc;

      *p++ = NEIGHBORS;

      {
        NODEINFO ni;
        u8_t nb[ZW_MAX_NODES / 8];
        u8_t *length;

        *p=0;
        length = p;
        p++;

        ZW_GetRoutingInfo_old(pData[2], nb, FALSE, FALSE);

        for (i = 0; i < ZW_MAX_NODES; i++)
        {
          if (rd_node_exists(i + 1) && BIT8_TST(i, nb))
          {
            *length += 2;
            ZW_GetNodeProtocolInfo(i + 1, &ni);

            *p++ = i + 1; //NodeID

            if (ni.reserved & ZWAVE_NODEINFO_BAUD_200K)
            {
              *p = IMA_NODE_SPEED_200;
            } else if (ni.reserved & ZWAVE_NODEINFO_BAUD_100K)
            {
              *p = IMA_NODE_SPEED_100;
            } else if (ni.capability & ZWAVE_NODEINFO_BAUD_40000)
            {
              *p = IMA_NODE_SPEED_40;
            } else if (ni.capability & ZWAVE_NODEINFO_BAUD_9600)
            {
              *p = IMA_NODE_SPEED_96;
            }

            if ((ni.capability & NODEINFO_ROUTING_SUPPORT) && (ni.capability & NODEINFO_LISTENING_SUPPORT) ) {
              *p |= IMA_NODE_REPEATER; //Is repeater
            }
            p++;
          }
        }
      }

      *p++ = TANSMISSION_TIME_SUM;
      *p++ = 4;
      *p++ = (ima->mts>>24) & 0xFF;
      *p++ = (ima->mts>>16) & 0xFF;
      *p++ = (ima->mts>> 8) & 0xFF;
      *p++ = (ima->mts>> 0) & 0xFF;

      *p++ = TANSMISSION_TIME_SUM2;
      *p++ = 4;
      *p++ = (ima->mts2>>24) & 0xFF;
      *p++ = (ima->mts2>>16) & 0xFF;
      *p++ = (ima->mts2>> 8) & 0xFF;
      *p++ = (ima->mts2>> 0) & 0xFF;

      ZW_SendDataZIP(c,pData,(u8_t)(p - pData),0);
    }
    break;

    case STATISTICS_CLEAR:
      ima_reset();
      break;
    default:
      return COMMAND_NOT_SUPPORTED;
      break;
  }
  return COMMAND_HANDLED;
}

'''


class IMAClient:
    def __init__(self):
        pass

    def get_ima_stats_for_node(self, node_id):
        # #define COMMAND_CLASS_NETWORK_MANAGEMENT_INSTALLATION_MAINTENANCE 0x67
        # PRIORITY_ROUTE_SET:
        # PRIORITY_ROUTE_GET
        # STATISTICS_GET: 0x04
        # CC_InstallationAndMaintenance.c
        zip_client = ZipClient()
        print("Requesting IMA")
        node_id_hex = '%02x' % node_id
        print (node_id_hex)
        packet = binascii.unhexlify('23028050a400006704' + node_id_hex)

        response = zip_client.send_sync_request_to_ctrl(packet)
        zip_client.stop()
        return self.parse_ima_response(response)

    def get_ima_stats_for_network(self,publish_result=True):
        req_topic = "pt:j1/mt:cmd/rt:ad/rn:zw/ad:1"
        resp_topic = "pt:j1/mt:evt/rt:ad/rn:zw/ad:1"
        msg = Message(service="zwave-ad", msg_type="cmd.network.get_all_nodes", value=None, value_type=ValueType.NULL.value)
        mq_client = MqttTransport()
        mq_client.connect()
        node_list = mq_client.send_request(req_topic, msg, resp_topic=resp_topic,resp_service="zwave-ad", resp_msg_type="evt.network.all_nodes_report")
        #      [{"address": "1",
        #       "hash": "zw_0_1_1",
        #       "power_source": "ac",
        #       "status": "UP"}]
        result = []
        for node in node_list.value :
            node_report = self.get_ima_stats_for_node(int(node["address"]))
            result.append(node_report)

        if publish_result :
            msg = Message(service="zwave-ad", msg_type="evt.zwnetstats.full_report", value=result, value_type=ValueType.OBJECT.value)
            mq_client.publish(resp_topic,msg)

        mq_client.stop()
        return result

    def reset_stats(self):
        zip_client = ZipClient()
        print("Resetting IMA stats")
        packet = binascii.unhexlify('23028050a400006706')
        zip_client.send_cmd_to_controller(packet)
        time.sleep(2)

    def get_lwr(self, node_id):
        zip_client = ZipClient()
        print("Requesting IMA")
        node_id_hex = '%02x' % node_id
        packet = binascii.unhexlify('23028050a400006702' + node_id_hex)
        data = zip_client.send_sync_request_to_ctrl(packet)
        zip_client.stop()
        result = {"node_id": 0, "has_lwr": False, "r1": 0, "r2": 0, "r3": 0, "r4": 0, "link_sp": 0}

        if data[0] == COMMAND_CLASS_NETWORK_MANAGEMENT_INSTALLATION_MAINTENANCE and data[1] == PRIORITY_ROUTE_REPORT:
            result["node_id"] = data[2]
            result["has_lwr"] = (data[3] == 1)
            result["r1"] = data[4]
            result["r2"] = data[5]
            result["r3"] = data[6]
            result["r4"] = data[7]
            try:
                result["link_sp"] = LINK_SPEED_MAP[data[8]]
            except:
                result["link_sp"] = data[8]

        return result

    def parse_ima_response(self, data):
        r = {}
        if data[0] == COMMAND_CLASS_NETWORK_MANAGEMENT_INSTALLATION_MAINTENANCE and data[1] == STATISTICS_REPORT:
            # 67 05 05(nodeid)
            #  01 01 0d
            #  03 01 00
            #  00 01 01
            #  02 08   01 83  06 03   0a 83   0b 83
            #
            #
            # 04 04 00 00 00 a3050400000805
            node_id = data[2]
            print("<IMAClient> IMA report from node " + str(node_id))
            r["node_id"] = data[2]
            r["tx_count"] = data[5]
            r["err_count"] = data[8]
            r["route_change"] = data[11]
            r["lwr"] = self.get_lwr(node_id)
            node_count = int(data[13] / 2)
            print("<IMAClient> NB count: " + str(node_count))
            r["nb_info"] = []
            for i in range(node_count):
                nb_info = {"node_id": 0, "link_sp": 0, "is_rep": False}
                node_info = data[14 + (i * 2):(14 + (i * 2)) + 2]
                print("<IMAClient> NB node: " + str(node_info[0]))
                nb_info["node_id"] = int(node_info[0])
                flag = node_info[1]
                speed = flag & ~(1 << 7)
                print("Speed = ", speed)
                nb_info["link_sp"] = LINK_SPEED_MAP[speed]
                if flag & IMA_NODE_REPEATER:
                    nb_info["is_rep"] = True

                r["nb_info"].append(nb_info)

            return r
        else:
            print("<IMAClient> Unknown packet")


if __name__ == "__main__":
    packet = binascii.unhexlify('67050501010d0301000001010208018306030a830b830404000000a3050400000805')
    print(IMAClient.parse_ima_response(packet))
