import binascii

from ziptools.zip_client import ZipClient
import ziptools.zip_client as zclient
from fimp.message import Message,ValueType
from fimp.mqtt_transport import MqttTransport
import time
# The test runs over FIMP protocol over MQTT.
# It sends cmd.basic.get_report command to every ac powered node and expects reports withing given time.
# at the end it generates report and publishes it as fimp message .


class NetworkTester:
    def __init__(self):
        pass

    def ping_node(self,node_id,number_of_pings):
        zip_client = ZipClient(auto_connect=False)
        packet = binascii.unhexlify('230280d0ab000005840200000000')
        success = 0
        for i in range(number_of_pings):
            response = zip_client.send_cmd_to_node(node_id,packet)
            print("Response = "+str(response))
            if response == zclient.ZIP_FRM_ACK_RESP:
                print("Node is alive")
                success +=1
            else:
                print("Node is not reachable")
        zip_client.stop()
        return success

    def ping_all_nodes(self,publish_result=True):
        req_topic = "pt:j1/mt:cmd/rt:ad/rn:zw/ad:1"
        resp_topic = "pt:j1/mt:evt/rt:ad/rn:zw/ad:1"
        msg = Message(service="zwave-ad", msg_type="cmd.network.get_all_nodes", value=None, value_type=ValueType.NULL.value)
        mq_client = MqttTransport()
        mq_client.connect()
        node_list = mq_client.send_request(req_topic, msg, resp_topic=resp_topic,resp_service="zwave-ad", resp_msg_type="evt.network.all_nodes_report")
        result = []
        for node in node_list.value :
            wakeup_int = 0
            if "wakeup_int" in node :
                wakeup_int = int(node["wakeup_int"])
            if node["power_source"] == "ac" or wakeup_int == -1:
                node_id = int(node["address"])
                r = self.ping_node(node_id,1)
                status = "OK" if r == 1 else "NO_RESP"
            else:
                print("Battery powered device is skipped.")
                status = "NO_PING"
            result.append({"address":node["address"],"status":status})
        if publish_result:
            msg = Message(service="zwave-ad", msg_type="evt.zwnetstats.net_ping_report", value=result, value_type=ValueType.OBJECT.value)
            mq_client.publish(resp_topic,msg)
        mq_client.stop()

    def run_network_cleanup(self):
            # {
            #     "serv": "zwave-ad",
            #     "type": "cmd.thing.delete",
            #     "val_t": "str_map",
            #     "val": {
            #                "address": "11",
            #                "stop": ""
            #            },
            #     "props": null,
            #     "tags": null
            #     }
            req_topic = "pt:j1/mt:cmd/rt:ad/rn:zw/ad:1"
            resp_topic = "pt:j1/mt:evt/rt:ad/rn:zw/ad:1"
            msg = Message(service="zwave-ad", msg_type="cmd.network.get_all_nodes", value=None, value_type=ValueType.NULL.value)
            mq_client = MqttTransport()
            mq_client.connect()
            node_list = mq_client.send_request(req_topic, msg, resp_topic=resp_topic,resp_service="zwave-ad", resp_msg_type="evt.network.all_nodes_report")
            #      [{"address": "1",
            #       "hash": "zw_0_1_1",
            #       "power_source": "ac",
            #       "status": "UP"}]


            # success {
            #   "ctime": "2018-04-05T11:29:13+0200",
            #   "props": {},
            #   "serv": "zwave-ad",
            #   "tags": [],
            #   "type": "evt.thing.exclusion_report",
            #   "val": {
            #     "address": "2"
            #   },
            #   "val_t": "object"
            # }

            # {
            #     "ctime": "2018-04-05T11:26:41+0200",
            #     "props": {
            #         "msg": "Device can't be deleted.",
            #         "src": "exclusion"
            #     },
            #     "serv": "zwave-ad",
            #     "tags": [],
            #     "type": "evt.error.report",
            #     "val": "NET_NODE_REMOVE_FAILED",
            #     "val_t": "string"
            # }

            for node in node_list.value :
                if node["hash"]=="zw_0_0_0" and node["power_source"]=="ac":
                    if self.ping_node(int(node["address"]),1) == 0 :
                        val = {"address":node["address"],"stop":""}
                        del_msg = Message(service="zwave-ad", msg_type="cmd.thing.delete", value=val, value_type=ValueType.STR_MAP.value)
                        print("Removing node "+node["address"])
                        ret_val = mq_client.send_request(req_topic, del_msg, resp_topic=resp_topic,resp_service="zwave-ad", resp_msg_type="evt.thing.exclusion_report",timeout=15)
                        if ret_val:
                            print("Node %s was deleted "%ret_val.value["address"])
                        else :
                            print("Node can't be deleted")

