import socket

# The class implement z-wave client over UDP
import threading

import binascii

COMMAND_CLASS_ZIP = 0x23
COMMAND_ZIP_PACKET = 0x02

# Byte-2 of Z/IP frame bit definitions
ZIP_FRM_ACK_REQ  = 0x80    # request receiver to send back an ACK
ZIP_FRM_ACK_RESP = 0x40    # ACK response to the previous frame
ZIP_FRM_NACK_RESP = 0x20   # Previously sent Z/IP packet didn't reach destination host
ZIP_FRM_WAIT_RESP = 0x10   # The destination host may have a long response time
ZIP_FRM_Q_FULL_RESP = 0x08 # Queue is full
ZIP_FRM_OP_ERR_RESP = 0x04 # Critical option is unrecognized

'''
from zip_frame.h
/** Byte-3 of Z/IP frame bit definitions */
#define ZIP_FRM_HDR_EXT     0x80    ///< header extension included
#define ZIP_FRM_ZWAVE_CMD   0x40    ///< Z-wave command included
#define ZIP_FRM_MORE_INFO   0x20    ///< more information
#define ZIP_FRM_SECURE      0x10    ///< Z/IP packet is secure

/** Flag bit-mask definition for Z/IP frame */
#define ZIP_FLAG_SECURE         0x01    ///< frame is secure
#define ZIP_FLAG_ACK_REQ        0x02    ///< set the Z/IP frame ACK request bit
#define ZIP_FLAG_ACK_RESP       0x04    ///< set the Z/IP frame ACK response bit
#define ZIP_FLAG_NACK_OPT_RESP  0x08    ///< set the Z/IP frame NACK response bit and "option error" bit
#define ZIP_FLAG_HDR_EXT        0x10    ///< set the Z/IP frame header extension included bit
#define ZIP_FLAG_ENCAP_INFO     0x20    ///< frame contains Z-wave encapsulation format info
#define ZIP_FLAG_MORE_INFO      0x40    ///< set the Z/IP frame "more information" bit

/** Z/IP frame option types */
#define ZIP_OPT_EXP_DELAY   0x01    ///< Expected delay
#define ZIP_OPT_MAINTN_GET  0x02    ///< Installation and maintenance get
#define ZIP_OPT_MAINTN_RPT  0x03    ///< Installation and maintenance report
#define ZIP_OPT_ENCAP_FMT   0x04    ///< Encapsulation format info

/** Z/IP encapsulation format information bitmask */
#define ZIP_ENCAP_FMT_INFO_CRC16   0x0001   ///< CRC-16

'''

ZIP_FRM_ZWAVE_CMD = 0x40   #

ZIP_FRM_HDR_EXT = 0x80
ZIP_FRM_HDR_SIZE = 7


class ZipClient:
    port_num = 4123
    ip_addr = "fd00:aaaa::3"
    response_data = None
    t1 = None
    is_running = True

    def __init__(self,auto_connect=True):
        self.socket = None
        self.last_ack_response = None
        self.event = threading.Event()
        self.ack_event = threading.Event()
        if auto_connect :
            self.init_socket()
        # close the socket

    def init_socket(self):
        if self.socket is None:
            # SOCK_DGRAM specifies that this is UDP
            self.socket = socket.socket(socket.AF_INET6, socket.SOCK_DGRAM, 0)
            # connect the socket, think of it as connecting the cable to the address location
            self.socket.connect((self.ip_addr, self.port_num))
            self.t1 = threading.Thread(target=self.receive_frame)
            self.t1.daemon = True
            self.t1.start()

    def stop(self):
        # close the socket
        self.socket.close()
        self.is_running = False

    def receive_frame(self):
        # 23 02 00 c0 24 00 00 05 84 02 00 00 6705100101000301000001000200040400000000050400000000
        while self.is_running:
            print('\n Waiting for new message')
            try :
                data, address = self.socket.recvfrom(4096)
            except:
                print("Receive error.")
                continue

            print('Received {} bytes from {}'.format(len(data), address))
            print(binascii.hexlify(data))
            if data[0] == COMMAND_CLASS_ZIP and data[1] == COMMAND_ZIP_PACKET:
                if data[2] & ZIP_FRM_ACK_RESP:
                    print("ACK frame")
                    self.last_ack_response = ZIP_FRM_ACK_RESP
                    self.ack_event.set()
                    continue
                elif data[2]& ZIP_FRM_NACK_RESP:
                    print("NACK frame")
                    self.last_ack_response = ZIP_FRM_NACK_RESP
                    self.ack_event.set()
                    continue
                elif data[2]& ZIP_FRM_WAIT_RESP:
                    print("WAIT frame")
                    self.last_ack_response = ZIP_FRM_WAIT_RESP
                    self.ack_event.set()
                    continue

                hdr_ext_len = 0
                if data[3] & ZIP_FRM_HDR_EXT:
                    hdr_ext_len = data[7];

                if data[3] & ZIP_FRM_ZWAVE_CMD:

                    self.response_data = data[ZIP_FRM_HDR_SIZE+hdr_ext_len:]
                    self.event.set()
            else :
                print("NOT ZIP frame")
                print(binascii.hexlify(data))

    def send_cmd_to_node(self, node_id, packet):
        self.ip_addr = 'fd00:bbbb::00%x' % node_id
        self.init_socket()
        # SOCK_DGRAM specifies that this is UDP
        # soc = socket.socket(socket.AF_INET6, socket.SOCK_DGRAM, 0)
        # connect the socket, think of it as connecting the cable to the address location
        # self.socket.connect((self.ip_addr, self.port_num))
        print("Sending to node :"+self.ip_addr)
        self.ack_event.clear()
        self.socket.send(packet)
        if self.ack_event.wait(60):
            return self.last_ack_response
        else:
            return self.ZIP_FRM_NACK_RESP


    def send_cmd_to_controller(self, packet):
        self.socket.send(packet)

    def send_sync_request_to_ctrl(self, packet):
        self.event.clear()
        self.send_cmd_to_controller(packet)
        if self.event.wait(120):
            print(binascii.hexlify(self.response_data))
            return self.response_data
        else:
            return None

