import argparse
from ziptools.ima_client import IMAClient
from ziptools.network_tester import NetworkTester
from fimp.mqtt_transport import MqttTransport
import json
import signal
import threading

proc_event = threading.Event()


def msg_handler(topic, address, fimp_msg):
    cmd = fimp_msg.msg_type
    print("New mqtt command" + cmd)
    if cmd == "cmd.zwnetstats.get_full_report" :
        imac = IMAClient()
        result = imac.get_ima_stats_for_network()
    elif cmd == "cmd.zwnetstats.reset":
        imac = IMAClient()
        imac.reset_stats()
    elif cmd == "cmd.zwnetstats.net_ping":
        net_tester = NetworkTester()
        r = net_tester.ping_all_nodes()
        print(r)
    elif cmd == "cmd.network.cleanup":
        net_tester = NetworkTester()
        net_tester.run_network_cleanup()


def signal_handler_function(signum, frame):
    proc_event.set()


if __name__ == "__main__":
    signal.signal(signal.SIGTERM, signal_handler_function)
    parser = argparse.ArgumentParser()
    parser.add_argument('-c', action='store', dest='cmd',help='Command name,supported values:get-ima-node,get-ima-net,test-network,get-lwr-node,ping,clear-stats,ping-net,net-cleanup,run-proc')
    parser.add_argument('-a', action='store', dest='address',help='Command name,supported values:get-ima,test-network')
    parser.add_argument('-n', action='store', dest='count',help='Number of iterations.')

    res = parser.parse_args()
    if res.cmd == "get-ima-node":
        imac = IMAClient()
        result = imac.get_ima_stats_for_node(int(res.address))
        print(json.dumps(result))
    elif res.cmd == "get-ima-net":
        imac = IMAClient()
        result = imac.get_ima_stats_for_network()
        print(json.dumps(result))
    elif res.cmd == "get-lwr-node":
        imac = IMAClient()
        result = imac.get_lwr(int(res.address))
        print(json.dumps(result))
    elif res.cmd == "clear-stats":
        imac = IMAClient()
        imac.reset_stats()
    elif res.cmd == "ping":
        net_tester = NetworkTester()
        count = int(res.count)
        r = net_tester.ping_node(int(res.address),count)
        r = int(r)
        success_rate = int((r*100)/count)
        print("Ping result = %d (%d of %d) "%(success_rate,r,count))
    elif res.cmd == "ping-net":
        net_tester = NetworkTester()
        r = net_tester.ping_all_nodes()
        print(r)
    elif res.cmd == "net-cleanup":
        net_tester = NetworkTester()
        net_tester.run_network_cleanup()
    elif res.cmd == "run-proc":
        mq_transport = MqttTransport()
        mq_transport.set_message_handler(msg_handler)
        mq_transport.connect()
        mq_transport.subscribe("pt:j1/mt:cmd/rt:ad/rn:zw/ad:1")
        proc_event.wait()
        mq_transport.stop()
