import argparse
import json
import os
import sqlite3
import shutil

class ZwareNetwork:
    jnet = None
    prod_templates_dir = ""
    def load_network_file(self,file_name):
        self.jnet = json.load(open(file_name))
        self.file_name = file_name

    def save_network_to_file(self,file_name):
         json.dump(self.jnet,open(file_name,"w"))

    def delete_node(self,node_id):
        print ("nodes before = %d"%len(self.jnet["nodes"]))
        for node in self.jnet["nodes"] :
            if node["nodeid"] == int(node_id) : 
                self.jnet["nodes"].remove(node)
                break
        print ("nodes after = %d "%len(self.jnet["nodes"]))
    
    def add_node(self,node_id,product_hash):
        template = self.load_product_template(product_hash,node_id)
        self.jnet["nodes"].append(template)
        return template["alias"]
    
    def replace_node_with_template(self,node_id,template_name):
        self.delete_node(node_id)
        self.add_node(node_id,template_name)

    def load_product_template(self,template_name,node_id):
        template_name = template_name.replace("zw_","")
        print("Loading template %s"%template_name)
        try:
            template = json.load(open(os.path.join(self.prod_templates_dir,template_name+".json")))
        except:
            print("Can't find template in main template folder.Trying cache")
            try:
                template = json.load(open(os.path.join(self.prod_templates_dir,"cache",template_name+".json")))
            except:
                print("Can't find product in cache folder.Loadin UNKNOWN product template")
                template = json.load(open(os.path.join(self.prod_templates_dir,"unknown_product.json")))   
        template["nodeid"] = int(node_id)
        return template    

    def is_prod_id_ok(self,node_id):
        print ("NodeId = ", node_id)
        for node in self.jnet["nodes"] :
            if node["nodeid"] == node_id:
                print ("Vid = ", node["vtype"])
                if node["vid"] != 0 and node["vtype"] != 0 : 
                    print("Node is ok")
                    return True
        else:
            return False

    def get_list_of_nodes_without_prod_info(self):
        result  = []
        for node in self.jnet["nodes"] :
            if self.is_prod_id_ok(node["nodeid"]) == False:
                result.append(node["nodeid"])
        return result                       

def restore_network_file_from_vinculum(zware_net,vinc_db_file):
    all_reports = []
    with sqlite3.connect(vinc_db_file) as db:
        cur = db.cursor()
        cur.execute("SELECT thingId, inclusionReport FROM Thing ")
        for prop in cur : 
            incl_rep = json.loads(prop[1])
            all_reports.append(incl_rep)
            node_id = incl_rep["val"]["address"]
            prod_hash = incl_rep["val"]["product_hash"]
            if not zware_net.is_prod_id_ok(int(node_id)):
                print("---------Recovering node id = %s "%node_id) 
                zware_net.replace_node_with_template(node_id,prod_hash)
                print("---------Done----------------------------")
            else :
                print("---------Node = %s is GOOD , no recovery-"%node_id)    
        

def load_missing_templates(legacy_network_file, zware_net,result_file):
        legacy_net = json.load(open(legacy_network_file))
        print("Number of nodes in legacy network = ", len(legacy_net["devices"]))
        missing_prod_info_nodes = zware_net.get_list_of_nodes_without_prod_info()
        print("--- Zware network--------")
        print("  Nodes without product info : ",missing_prod_info_nodes)
        for mnode in missing_prod_info_nodes:
            dev_hash = ""
            for device in legacy_net["devices"]:
                if device["nodeId"] == mnode :
                    security_postfix = ""
                    if device["secure"]: 
                        security_postfix = "_S0"
                    if device["fimpHash"] == None :
                        dev_hash = ""
                    else :    
                        dev_hash = device["fimpHash"].replace("zw_","")
                        dev_hash = dev_hash+security_postfix
            if dev_hash:
                print("  Node doesn't have product info ,nodeId = ",str(mnode))
                print("  Replacing node structure with data from template with hash:  ",dev_hash)
                zware_net.replace_node_with_template(mnode,dev_hash)
        zware_net.save_network_to_file(result_file)             
               


if __name__ == "__main__":
    print("----Zware network file tools----")
    parser = argparse.ArgumentParser(description='Process some integers.')
    parser.add_argument("-o","--operation",choices=["add_node","delete_node","load_missing_templates","recover_from_vinc"],required=True,action='store', dest='operation')
    parser.add_argument("-n","--nodes",required=False,action='store', dest='nodes_str',help="Comma separated list of nodes")
    parser.add_argument("-p","--products",action='store', dest='products_str',help="Comma separated list of products")
    parser.add_argument("-z","--zwarenet",action='store', dest='zware_net_file',help="Full path to zware_net.json",default="/var/lib/futurehome/zwave-ad/zware_net.json")
    parser.add_argument("-vdb","--vinc_db",action='store', dest='vinc_db_file',help="Full path vinculum sqlitedb",default="/var/lib/futurehome/vinculum/storage.sqlite")
    
    cli_params = parser.parse_args()
    zware_net_file_path = "/var/lib/futurehome/zwave-ad/zware_net.json"
    # zware_net_file_path = "./zware_network_generator/examples/bad_zware_net.json"
    # fixed_net_file_path = "./zware_network_generator/examples/fixed_zware_net.json"
    # legacy_znet_file = "./zware_network_generator/examples/cust_1.json"
    legacy_znet_file = "/var/spool/futurehome/migration/zwave.json"
    # fixed_net_file_path = "/var/lib/futurehome/zwave-ad/fixed_zware_net.json"

    vinc_db_file = "/var/lib/futurehome/vinculum/storage.sqlite"
    vinc_db_file_copy = "/tmp/storage.sqlite"
    # ---------- Test ---------------------------------
    
    # vinc_db_file = "./testdata/storage.sqlite"
    # vinc_db_file_copy = "./testdata/storage_copy.sqlite"

    # --------------------------------------------------    
    zware_net = ZwareNetwork()
    # zware_net.prod_templates_dir = "../products"
    zware_net.prod_templates_dir = "/var/lib/futurehome/zwave-ad/products"
    zware_net_file_path = cli_params.zware_net_file
    # ---------test 
    # zware_net.prod_templates_dir = "../products"
    # zware_net_file_path = "./testdata/zware_net.json"
    # -------------
    zware_net.load_network_file(zware_net_file_path)
    print("Operation = "+cli_params.operation)
        
    if cli_params.operation == "add_node" : 
        nodes = cli_params.nodes_str.split(",")
        print("Adding nodes to network:")
        products = cli_params.products_str.split(",")
        if len(nodes) == len(products):
            i = 0
            for node in nodes:
                alias = zware_net.add_node(node,products[i])
                print(" node = %s , product = %s"%(node,alias))
                i+=1

        zware_net.save_network_to_file(zware_net_file_path)        

    elif cli_params.operation == "delete_node":
        nodes = cli_params.nodes_str.split(",")
        print("Removing nodes to network:")
        for node in nodes:
            zware_net.delete_node(int(node))
            zware_net.save_network_to_file(zware_net_file_path)

    elif cli_params.operation == "load_missing_templates":
        print("Loading missing templates to zware_net") 
        load_missing_templates(legacy_znet_file, zware_net,zware_net_file_path)
    elif cli_params.operation == "recover_from_vinc":
        print("Recovering network file from vinculum db")
        shutil.copyfile(vinc_db_file,vinc_db_file_copy)
        shutil.copyfile(zware_net_file_path,zware_net_file_path+".back")
        restore_network_file_from_vinculum(zware_net,vinc_db_file_copy)
        zware_net.save_network_to_file(zware_net_file_path)


# cd /usr/share/futurehome/zwave-ad/scripts        
# sudo python zware_network_file_tools.py -o delete_node -n 65 
# sudo python zware_network_file_tools.py -o add_node -n 65 -p 411_3_514         


