#!/usr/bin/python3

import json
import os.path


def load_product_template(templates_dir,template_name,node_id):
    template = json.load(open(os.path.join(templates_dir,template_name+".json")))
    template["nodeid"] = int(node_id)
    return template


def generate_network_file(legacy_network_file, zware_products_dir, output_file):
    legacy_net = json.load(open(legacy_network_file))
    print("Number of nodes = ", len(legacy_net["devices"]))

    zware_net = {"group_1_node_id":0,"homeid":0,"nodes":[]}

    for device in legacy_net["devices"]:
            security_postfix = ""
            if device["secure"]: 
              security_postfix = "_S0"
            if device["fimpHash"] == None :
                dev_hash = "unknown_product"
                print("Loading unknown product template ")
            else :    
                dev_hash = device["fimpHash"].replace("zw_","")
                dev_hash = dev_hash+security_postfix
            
            try:
                device_template = load_product_template(zware_products_dir,dev_hash,device["nodeId"])
                zware_net["nodes"].append(device_template)
                print("Dev info added for NodeId = "+str(device["nodeId"]))
            except Exception as ex:
                print("Template for product "+dev_hash+" failed to load. Error: "+str(ex))
       
    return zware_net

if __name__ == "__main__":
    print("----Generating Z-Ware network definition file----")
    # ---------------Configs-------------------

    # Location of Z-Wave Network information file produced by legacy core
    legacy_zwave_network_file = "/var/spool/futurehome/migration/zwave.json"
    # legacy_zwave_network_file = "./zware_network_generator/examples/core_zwave_net.json"
    # Location of product templates
    zware_product_templates_dir = "/var/lib/futurehome/zwave-ad/products"
    # zware_product_templates_dir = "../products"
    # Location of output file .
    output_file = "/var/lib/futurehome/zwave-ad/zware_net.json"
    # output_file = "./zware_network_generator/zware_net.json"
    # ---------------END-----------------------
    zwnet = generate_network_file(legacy_zwave_network_file, zware_product_templates_dir, output_file)
    json.dump(zwnet,open(output_file,"w"))
    print("----Done---- ")
